function [J, g] = cg_l2(imr, par)
%
% Fonction calculant le critre quadratique utilis en nrestauration d'images
% ainsi que son gradient
%
% Paramtres d'entre
%  imr : valeur courante de l'estime de l'image restaure vecteur (MN, 1)
%  par : structure comportant les champs suivants
%  im : image  restaurer. Tableau rel de taille (M, N)
%  h : PSF de la fonction de dgradation
%  lambda1 : paramtre de rgularisation, terme de rappel zro
%  lambda2 : paramtre de rgularisation, diffrences premires
%  delta : facteur d'chelle du terme semi quadratique
%          
% Paramtres de sortie
%   J : critre
%   g : gradient de J
%
%  f : image restaure, de mme taille et de mme type que g
%
%    Copyright (C) 2008-2014  Yves Goussard
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.

im = par.im;
h = par.h;
lambda1 = par.lambda1;
lambda2 = par.lambda2;


[M, N] = size(im);

if (M*N ~= length(imr))
    fprintf('cg_l2l2 -- Erreur : tailles de im et imr incohrentes')
    return
end

imr = reshape(imr, M, N);

% Valeur du critre

ht = flipdim(flipdim(h, 1), 2);
predic = conv2(imr, h, 'same');
v2 = im - predic;

ux = imr(1:M-1, :) - imr(2:M, :);
uy = imr(:, 1:N-1) - imr(:, 2:N);

J = v2(:)' * v2(:) + ...
    lambda1 * sum(imr(:).^2) + ...
    lambda2 * (sum(ux(:).^2) + sum(uy(:).^2));

% Valeur du gradient

B0 = 2 * imr;

bx = zeros(size(imr));
bx(1:M-1, :) = ux;
Bx = 2 * (bx - circshift(bx, [1, 0]));

by = zeros(size(imr));
by(:, 1:N-1) = uy;
By = 2 * (by - circshift(by, [0, 1]));

g = - 2 * conv2(v2, ht, 'same') + lambda1 * B0 + lambda2 * (Bx + By);

g = g(:);

