% Restauration d'une image dgrade par un filtrage linaire et
% l'addition d'un bruit gaussien
%
%    Copyright (C) 2008-2014  Yves Goussard
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.


im = imread('FigP0520(blurred-heart).tif');
classin = class(im);
z = double(im);

figure(1), clf, imshow(im)
title('Image floue', 'FontSize', 14)
disp('Tapez <RC> pour poursuivre')
pause

P = 6
sigma = P/2.5;

t = -P:P;
[T1, T2] = ndgrid(t', t);

h = 1/(2*pi*sigma^2) * exp(- (T1.^2 + T2.^2) / (2*sigma^2));

% z = conv2(x, h, 'same') + 4 * randn(size(im));

figure(2), clf, imagesc(h), colormap gray, colorbar, axis image
title('PSF', 'FontSize', 14)
disp('Tapez <RC> pour poursuivre')
pause

[M, N] = size(z);

lambda1 = 1e-02
lambda2 = 1e-02

x0 = z(:);

par = struct('im', z, 'h', h, 'lambda1', lambda1, ...
    'lambda2', lambda2);
opts = zeros(1, 9);
[xr, info, perf] = conj_grad('cg_l2', par, x0, opts);

eval(['xhl2 = ', classin, '(reshape(xr(:, end), M, N));'])

figure(3), clf, imshow(xhl2)
title('Restauration, pnalisation quadratique', 'FontSize', 14)

lambda1 = 1e-02
lambda2 = 1e-02

x0 = z(:);
delta = max(z(:))/1000;

par = struct('im', z, 'h', h, 'lambda1', lambda1, ...
    'lambda2', lambda2, 'delta', delta);
opts = zeros(1, 9);
[xr, info, perf] = conj_grad('cg_l2l1', par, x0, opts);

eval(['xhl2l1 = ', classin, '(reshape(xr(:, end), M, N));'])

figure(4), clf, imshow(xhl2l1)
title('Restauration, pnalisation L_2L_1', 'FontSize', 14)

